/* We need to know about HDF-EOS, of course... */
#include "HE5_HdfEosDef.h"

/* and the XML processing software (expat and SCEW) */
#include "scew/scew.h"

/* and stdio and stdlib, what the heck */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>

/* Trace code support.  TRACE((stderr, "format string", args...)) */
#ifdef TRACE_OUTPUT
#define TRACE(x) fprintf x
#define TRACEWRAP(x) x
#else
#define TRACE(x)
#define TRACEWRAP(x)
#endif

/* turn this on to enable malloc/free tracing */
#ifdef TRACE_MALLOC
#define malloc MYmalloc
#define calloc MYcalloc
#define free MYfree
#endif

/* support for simple and stupid error handling -
  if a call returns a bad status,
   just print the status, the statement that errored, and exit.  We'll
   have to do better in the future, of course...
*/

/* ANSI pre-processor macro substitution, gotta love it... */

#define SS(x) spit_status(x, #x "")


/* And declarations for stuff in other files (like support.c) */

#ifdef TRACE_MALLOC
void *MYmalloc(size_t size);
char *MYcalloc(size_t nmemb, size_t size);
void MYfree(void *ptr);
#endif

void spit_status(int stat, char const *call);
void fatal_error(char const *s1, char const *s2);

char *newstring(unsigned int len);
char *strclone(char const *in);
int regex_matches(char const *str, char const *pattern);
long careful_atol(char const *a, H5T_sign_t signp, int *bogus);
void end_name(char *namebuf);
char *next_name (char *namebuf);
int name_in_list(char const *name, char *namelist, int numnames);

void push_name(char const *type, char const *name);
void pop_name(void);
void print_name(FILE *outstream);
char const *current_name(void);
char const *current_type(void);

void bump_errors(void);
void bump_warnings(void);
int print_final_message(void);

/* and xmlsupport.c */
int check_element_name(scew_element const *elt, char const *name, int crap_out);
char const *get_attribute_string(scew_element const *elt, char const *name);
void print_scew_element (scew_element const *elt, FILE *out);

typedef struct valnode {
  struct valnode *next;
  char const *name;
  union {
    char const *strv;
    long intv;
    double floatv;
  } v;
} valnode;

void save_string(char const *name, char const *val);
void save_int(char const *name, long val);
void save_float(char const *name, double val);
char const *get_saved_string(char const *name);
long const *get_saved_int(char const *name);
double const *get_saved_float(char const *name);


/* and hdfeos-support.h */

/* the types of HDF-EOS5 objects we're going to mung */
enum he5_object_enum {
  he5_file, he5_swath, he5_geofield, he5_datafield};

typedef enum he5_object_enum he5_object;

int attribute_names(hid_t handle, he5_object htype, char const *field,
		    char *namebuf, unsigned long *bufsize);
herr_t attribute_info(hid_t handle, he5_object htype, char const *field,
		      char const *attname,
		      H5T_class_t *type, hsize_t *count, size_t *size, H5T_sign_t *sgn);
herr_t read_attribute(hid_t handle, he5_object htype, char const *field,
		      char const *attname, void *databuf);

hid_t read_in_type(hid_t type);
H5T_class_t type_info (hid_t type, unsigned int *size, int *signp);
